from coralillo.tests.test_engine import EngineTestCase
from coralillo.tests.test_events import EventTestCase
from coralillo.tests.test_fields import FieldTestCase
from coralillo.tests.test_models import ModelTestCase
from coralillo.tests.test_permissions import PermissionTestCase
from coralillo.tests.test_relations import RelationTestCase
from coralillo.tests.test_presentation import PresentationTestCase
from coralillo.tests.test_utils import UtilsTestCase
import unittest


if __name__ == '__main__':
    unittest.main()
